/******************************************************************************
  DecimalNotes.c   Makes highly accurate musical note frequencies from any xtal.
  Copyright; Sep 2012 Roman Black
  Compiler; MikroC or MikroC PRO
  Open source; use it for whatever you like, but please mention me as the
  original author and mention my web page; www.RomanBlack.com/High_Acc_Timing.htm

  PIC; 16F628A 16MHz HS xtal
  Description; This is based on my earlier code that makes highly accurate
  frequencies from 0.0001 Hz to 50000.0001 Hz. It uses a set of 12 constants
  for the exact notes, and they are divided by 2 etc to make lower octaves.
  Faster xtal speeds will make exactly the same note but with slightly less
  edge jitter (so gives more perfect squarewaves). These accurate notes
  may be useful for musical instrument tuning.
  
  Note! Make sure with PIC16 that total interrupt size is less than 90
  instructions (check MikroC View Statistics).
  This version is in 0.0001 Hz resolution.
  Minimum frequency; 0.0001 Hz
  Maximum frequency; 50000.0000 Hz (when using 40MHz osc)
  (Max freq is calculated by; TMR2 freq / 200)
  Compatibility; PIC16 and PIC18 both ok, requires TMR2.
******************************************************************************/
// set this number to your XTAL or HSPLL MHz value (same as MikroC project)
#define HZ 16000000   // 16MHz HS xtal = 16000000, 40MHz HSPLL = 40000000

// (Don't change this define! See "how_works.txt" for details.)
#define TOGGLE (((HZ * 5) / 2) * 5)   // is *12.5 without overflowing long

// User functions;
void start_octave_note(unsigned char octave, unsigned char note);
void start_midi_note(unsigned char mnote);
void stop_note(void);

// private variables;
volatile unsigned long accum;
volatile unsigned long freqHz;


//=============================================================================
//   INTERRUPT
//=============================================================================
void interrupt(void)
{
  //-------------------------------------------------------
  // This is the TMR2 interrupt, set to 100 timer ticks
  // Frequency generation is done by a combination of DDS accumulator
  // and a decimal bresenham subtraction that makes the value
  // in actual decimal fractions of Hz.
  // For more info see my page; www.RomanBlack.com/High_Acc_Timing.htm
  //-------------------------------------------------------
  PIR1.TMR2IF = 0;      // clear TMR2 roll flag
  accum += freqHz;      // decimal DDS addition into accumulator
  if(accum >= TOGGLE)
  {
    PORTB = (PORTB ^ 0x01);   // toggle RB0 output pin to make frequency
    accum -= TOGGLE;    // decimal bresenham subtraction, keep remainder
  }
}
//-----------------------------------------------------------------------------


//=============================================================================
//   START_OCTAVE_NOTE
//=============================================================================
void start_octave_note(unsigned char xoctave, unsigned char xnote)
{
  //-------------------------------------------------------
  // This creates the freq number in variable freqHz, which enables
  // the interrupt to automatically produce a highly accurate note.
  // Octave and notes are based on MIDI music standards;
  // lowest octave = 0 (A=27.5Hz), highest octave = 9 (A=14080 Hz)
  // notes are always 0-11 (C to B)
  //-------------------------------------------------------
  unsigned long nfreq;    // temp holds note frequency
  unsigned  int ndiv;     // temp holds octave divisor
  
  // The note is always C-B, so get freq of note in the highest octave
  if(xnote == 0)  nfreq =  83720181;  // C     8372.0181 Hz
  if(xnote == 1)  nfreq =  88698442;  // C#
  if(xnote == 2)  nfreq =  93972725;  // D
  if(xnote == 3)  nfreq =  99560635;  // D#
  if(xnote == 4)  nfreq = 105480818;  // E
  if(xnote == 5)  nfreq = 111753034;  // F
  if(xnote == 6)  nfreq = 118398215;  // F#
  if(xnote == 7)  nfreq = 125438539;  // G
  if(xnote == 8)  nfreq = 132897503;  // G#
  if(xnote == 9)  nfreq = 140800000;  // A    14080.0000 Hz (A 440Hz * 32)
  if(xnote == 10) nfreq = 149172404;  // A#
  if(xnote >= 11) nfreq = 158042656;  // B    15804.2656 Hz

  // For each octave down we must divide the note freq by 2,
  // but first for good accuracy we need to add half the divisor
  // so need to calc half the divisor
  if(xoctave > 9) xoctave = 9;      // safe limit the user selected octave!
  ndiv = 1;
  ndiv = (ndiv << (9 - xoctave));   // ndiv = the divisor
  ndiv >>= 1;                       // ndiv = half the divisor
  nfreq += ndiv;                    // add half the divisor before dividng down
  
  // do divison by right shifts for smaller code (doesn't need div32 library)
  while(xoctave > 9)
  {
    nfreq >>= 1;
    xoctave++;
  }

  // start playing the new note
  PIE1.TMR2IE = 0;      // stop old playback
  freqHz = nfreq;       // set the new freq to play
  accum = 0;
  PIE1.TMR2IE = 1;      // play it!
}
//-----------------------------------------------------------------------------


//=============================================================================
//   START_MIDI_NOTE
//=============================================================================
void start_midi_note(unsigned char mnote)
{
  //-------------------------------------------------------
  // This function starts playing a note based on MIDI note number 0-107.
  // There are 12 MIDI notes to each octave.
  //-------------------------------------------------------
  unsigned char moctave;      // temp, used to calc octave
  // Convert MIDI note 0-107 into octave 0-9, and note 0-11
  if(mnote > 107) mnote = 107;    // safe!
  moctave = 0;
  while(mnote >= 12)
  {
    moctave++;
    mnote -= 12;
  }
  // and play the note!
  start_octave_note(moctave,mnote);
}
//-----------------------------------------------------------------------------


//=============================================================================
//   STOP_NOTE
//=============================================================================
void stop_note(void)
{
  PIE1.TMR2IE = 0;    // stop the interrupt so note stops playing
  PORTB.F0 = 0;       // and set the output pin low to save power
                      // (check your port pin here and in the interrupt)
}
//-----------------------------------------------------------------------------


//=============================================================================
//   MAIN
//=============================================================================
void main(void)
{
  //-------------------------------------------------------
  // setup PIC 16F628A
  CMCON = 0x07;   // comparators OFF

  PORTA = 0b00000000;   // PORTA all outputs
  TRISA = 0b00000000;

  PORTB = 0b00000000;   // RB0 is freq output, rest of PORTB also outputs
  TRISB = 0b00000000;

  // setup TMR2 to roll every 100 ticks
  T2CON = 0b00000100;   // TMR2 ON, 1:1 prescale
  PR2 = (100-1);        // TMR2 int occurs every 100 ticks

  // enable interrupt here (and just leave TNR2 int running in this example)
  INTCON = 0b11000000;  // GIE = ON, PEIE = ON
  accum = 0;            // and clear the variable used for note timing

  //-------------------------------------------------------
  // example; to make a note just tell it the octave and note;
  start_octave_note(4,9);   // octave 4, note 9 (A 440 Hz)

  // example; to stop the note;
  //stop_note();

  // This is the main running loop/
  while(1)
  {
    //---------------------------------------------
    // your code goes here!



  }
}
//-----------------------------------------------------------------------------







